import sys
import ast
import symtable
sourceByLines = []
source = ""
for line in sys.stdin:
    sourceByLines.append(line)
    source += line
filename = sys.argv[1]
a = None
try:
    a = ast.parse(source, filename, "exec")
except UnicodeEncodeError:
    for i in range(len(source)):
        if ord(source[i]) > 127:
            replacement = " "
            if source[i].isalnum(): replacement = "_"
            source = source[:i] + replacement + source[i+1:]
    a = ast.parse(source, filename, "exec")
s = symtable.symtable(source, filename, "exec")

symtable_lookup = {}
def recurse_symtable(table):
    pos = table.get_lineno()
    assert pos not in symtable_lookup
    symtable_lookup[pos] = table
    for child in table.get_children():
        recurse_symtable(child)
recurse_symtable(s)

def recurse(f, node, stack = []):
    f(node, stack)
    stack.append(node)
    for child in ast.iter_child_nodes(node):
        recurse(f, child, stack)
    stack.pop()

def get_names(node):
    if not hasattr(node, "lineno"):
        return []
    if isinstance(node, ast.Name):
        return [node.id]
    elif hasattr(ast, "arg") and isinstance(node, ast.arg):
        return [node.arg]
    elif hasattr(node, "names"):
        return node.names
    elif hasattr(node, "name"):
        return [node.name]
    return []

def get_symbol(node, stack, name, global_only = False):
    lineno = -1
    if stack and hasattr(node, "lineno"):
        lineno = node.lineno
    elif stack == []:
        lineno = 0
    if lineno in symtable_lookup:
        table = symtable_lookup[lineno]
        symbol = None
        try:
            symbol = table.lookup(name)
        except:
            symbol = None
        if symbol != None:
            if symbol.is_global() and stack:
                return get_symbol(stack[-1], stack[0:-1], name, True)
            if symbol.is_free() or not symbol.is_local():
                symbol = None
            else:
                return symbol, table
    if stack:
        return get_symbol(stack[-1], stack[0:-1], name, global_only)
    return None, None

result = []

def is_identifier_char(s):
    if s == "": return False
    if hasattr(str, "isidentifier"): return ("_"+s).isidentifier()
    if s.isalnum(): return True
    s = s.replace("_", "")
    return s == "" or s.isalnum()

def findword(word, string, loc = 0):
    while loc < len(string):
        find = string.find(word, loc)
        if find == -1:
            return find
        loc = find+len(word)
        if find != -1 and ((find == 0 or not is_identifier_char(string[find-1])) and (find+len(word) == len(string) or not is_identifier_char(string[find+len(word)]))):
            return find
    return -1

def visit(node, stack):
    for name in get_names(node):
        symbol, symtable = get_symbol(node, stack, name)
        if symbol != None:
            col = node.col_offset
            type = None
            if node.lineno:
                line = sourceByLines[node.lineno-1]
                end = len(name)+col
                if line[col:end] != name or (end < len(line) and is_identifier_char(line[end])):
                    find = findword(name, line, col)
                    if find != -1: col = find
                loc = col+len(name)
                while loc < len(line) and (line[loc] == ' ' or line[loc] == '\t'):
                    loc += 1
                if loc < len(line) and line[loc] == '(':
                    type = "function"
            obj = {"id":name+"-"+str(symtable.get_id()), "location":str(node.lineno)+":"+str(col)+","+str(len(name))}
            if type != None:
                obj["type"] = type
            result.append(obj)

recurse(visit, a)
import json
print(json.dumps(result, indent=True))
